use std::fs::read_to_string;

use color_eyre::Result;
use itertools::Itertools;

fn initial(input: &str) -> (usize, usize) {
    let ranges = input.lines().map(|line| {
        line.split_terminator(',')
            .map(|range| {
                let (lower, upper): (usize, usize) = range
                    .split_terminator('-')
                    .map(|number| number.parse::<usize>().unwrap())
                    .take(2)
                    .collect_tuple()
                    .unwrap();
                lower..=upper
            })
            .take(2)
            .collect_tuple()
            .unwrap()
    });
    let containing = *ranges
        .clone()
        .map(|(first, second)| {
            first.contains(second.start()) && first.contains(second.end())
                || second.contains(first.start()) && second.contains(first.end())
        })
        .counts()
        .get(&true)
        .unwrap_or(&0usize);
    let overlaping = *ranges
        .map(|(first, second)| {
            first.contains(second.start())
                || first.contains(second.end())
                || second.contains(first.start())
                || second.contains(first.end())
        })
        .counts()
        .get(&true)
        .unwrap_or(&0usize);
    (containing, overlaping)
}

fn main() -> Result<()> {
    let input = read_to_string("input.txt")?;
    let (containing, overlaping) = initial(&input);
    println!(
        "{} pairs have containing ranges, {} ranges overlap",
        containing, overlaping
    );
    Ok(())
}
